/*
 * O(N) algorithm for planar graph 5-coloring 
 *
 * Algorithm is based on the following theorem:
 *
 * Every planar graph has:
 * a) a vertex of degree at most 4, or
 * b) a vertex of degree 5 adjacent to at least 4 vertices of degree <=11
 */

#include <algorithm>
#include <list>
#include <vector>
#include <cassert>
#include <cstdio>
#include <cstring>
#include <stack>

#define foreach(it,c) for (typeof(c.begin()) it=c.begin();it!=c.end();it++)

const int MAXN=100000;

using namespace std;

struct edge {  // edge vw
  int w;
  list<edge>::iterator wv;
};

list<edge> graph[MAXN]; // adjacency list
int deg[MAXN]; // degrees
list<int> deglist[MAXN-1]; // list of vertices of given degree
list<int> five11; // list of vertices of degree 5 adjacent to at least 4 vertices of degree <=11
list<int> *deglistptr[MAXN]; // pointer to degree list, where vertex resides
list<int>::iterator deglistit[MAXN]; // iterator in that list
int color[MAXN]; // vertex color
stack<int> order; // order of vertex coloring
vector<int> adj[MAXN]; // adjacent vertices of given vertex
int identified[MAXN][2]; // which nodes were identified

/* Tests whether v has at least 4 vertices of degree <=11 */
int ok5(int v)
{
  int sum=0;
  foreach(it,graph[v]) 
    sum+=deg[(*it).w]<=11;
  return sum>=4;
}

/* Inserts vertex in degree list */
void inserttodeg(int v)
{
  if (deg[v]==5 && ok5(v)) {
    five11.push_back(v);
    deglistit[v]=--five11.end();
    deglistptr[v]=&five11;
  } else {
    deglist[deg[v]].push_back(v);
    deglistit[v]=--deglist[deg[v]].end();
    deglistptr[v]=&deglist[deg[v]];
  }
}

/* Erases vertex from degree list */
void erasefromdeg(int v)
{
  (*deglistptr[v]).erase(deglistit[v]);
}

/* Adds an edge between v and w */
void addedge(int v, int w)
{
  edge e;
  e.w=w;
  graph[v].push_back(e);
  e.w=v;
  graph[w].push_back(e);
  graph[v].back().wv=--graph[w].end();
  graph[w].back().wv=--graph[v].end();

  if (deglistptr[v]!=NULL) erasefromdeg(v);  
  if (deglistptr[w]!=NULL) erasefromdeg(w);  
  deg[v]++; deg[w]++;
  inserttodeg(v);
  inserttodeg(w);
}

/* Erases node from graph */
void erasenode(int v)
{
  erasefromdeg(v);
  deglistptr[v]=NULL;
  deg[v]=0;
  foreach(it,graph[v]) {
    graph[(*it).w].erase((*it).wv);
    erasefromdeg((*it).w);
    deg[(*it).w]--;
    inserttodeg((*it).w);
    if (deg[(*it).w]==11) {
      foreach(it2,graph[(*it).w]) {
        if (deglistptr[(*it2).w]!=&five11 && deg[(*it2).w]==5 && ok5((*it2).w)) {
          erasefromdeg((*it2).w);
          inserttodeg((*it2).w);
        }
      }
    }
  }
  graph[v].clear();
}

/* Tests if there is vw edge */
int isedge(int v, int w)
{
  foreach(it,graph[v]) 
    if ((*it).w==w) return 1;
  return 0;
}

/* Identifies vertices u and w */
void identify(int u, int w)
{
  vector<int> adj;
  foreach(it,graph[u]) 
    adj.push_back((*it).w);
  foreach(it,graph[w]) 
    adj.push_back((*it).w);
  erasenode(u);
  erasenode(w);
  sort(adj.begin(),adj.end());
  adj.erase(unique(adj.begin(),adj.end()),adj.end());
  
  for (int i=0;i<adj.size();i++) 
    addedge(u,adj[i]);
}

/* Computes coloring */
void coloring()
{
  while (1) {
    int v=-1;
    for (int i=0;i<=4;i++)
      if (!deglist[i].empty()) {
        v=deglist[i].front();
        break;
      }

    if (v==-1) 
      if (!five11.empty())
        v=five11.front();

    if (v==-1) 
      break;

    // graph[v] has at most 5 elements -> O(1)
    foreach(it,graph[v]) {
      adj[v].push_back((*it).w);
    }

    if (deg[v]<=4) {
      // we are erasing vertex with degree <=4 -> O(1)
      erasenode(v);
    } else {
      int u=-1,w=-1;
      foreach(it,graph[v]) {
        for (typeof(it) it2=it;it2!=graph[v].end();it2++) {
          // we are checking for an edge between at most O(5^2) vertices
          // one isedge() call costs us O(deg(u)) where deg(u)<=11
          if (deg[(*it).w]<=11 && deg[(*it2).w]<=11 && (*it).w!=(*it2).w && !isedge((*it).w,(*it2).w)) {
            u=(*it).w; 
            w=(*it2).w;
            break;
          }
        }
        if (w!=-1) 
          break;
      }

      // u,w have degree <= 11 -> O(1)
      identify(u,w);
      identified[v][0]=u;
      identified[v][1]=w;
      erasenode(v);
    }

    order.push(v);

    // we are processing every node exactly once, so this cycle is O(N)
  }

  while (!order.empty()) {
    int used[5];
    int v=order.top();
    order.pop();
    memset(used,0,sizeof(used));
    if (adj[v].size()==5) 
      color[identified[v][1]]=color[identified[v][0]];
    for (int i=0;i<adj[v].size();i++) {
      used[color[adj[v][i]]]=1;
    }
    for (color[v]=0;used[color[v]];color[v]++)
      ;
  }
}

int main()
{
  int n,m;
  stdin = fopen("color.in", "r");
  stdout = fopen("color.out", "w");
  scanf("%d %d",&n,&m);

  for (int i=0;i<n;i++)
    inserttodeg(i);

  for (int i=0;i<m;i++) {
    int a,b;
    scanf("%d %d",&a,&b);
    a--; b--;
    assert(a!=b);
//    assert(!isedge(a,b));
    addedge(a,b);
  }

  memset(color,-1,sizeof(color));

  coloring();

  printf("%d",color[0]+1);
  for (int i=1;i<n;i++) { 
    printf(" %d",color[i]+1);
  }
  printf("\n");

  return 0;
}
